@@ -3,6 +3,7 @@ import math
 from typing import List, Optional
 
 import regex as re
+import stanza
 from lang_trans.arabic import buckwalter
 from pelecanus import PelicanJson
 
@@ -44,8 +45,7 @@ class Document:
             original_text: str,
             dominant_language_code: str,
             analysis_lang: str,
-            stanza_sentences: [list],
-            stanza_entities,
+            stanza_document: stanza.Document = None,
             entity_mapper: Optional[EntityMapper] = None,
             doc_path: str = "text_mlp",
             json_doc: dict = None,
@@ -64,10 +64,11 @@ class Document:
         self.entities_processed = False
 
         self.entity_mapper = entity_mapper
-        self.stanza_sentences = stanza_sentences
-        self.stanza_words = [word for sentence in self.stanza_sentences for word in sentence]
-        self.stanza_entities = stanza_entities
+        self.stanza_document = stanza_document
 
+        self.__stanza_sentences = []
+        self.__stanza_words = []
+        self.__stanza_entities = []
         self.__words = []
         self.__lemmas = []
         self.__pos_tags = []
@@ -76,7 +77,35 @@ class Document:
         self.__texta_facts: List[Fact] = []
 
         self.__handle_existing_facts()
-        self.words()
+
+        if self.stanza_document:
+            self.words()
+
+
+    @property
+    def stanza_sentences(self):
+        if not self.__stanza_sentences and self.stanza_document:
+            for sentence in self.stanza_document.sentences:
+                self.__stanza_sentences.append(sentence)
+        return self.__stanza_sentences
+
+
+    @property
+    def stanza_words(self):
+        if not self.__stanza_words and self.stanza_document:
+            for sentence in self.__stanza_sentences:
+                for word in sentence.words:
+                    self.__stanza_words.append(word)
+
+        return self.__stanza_words
+
+
+    @property
+    def stanza_entities(self):
+        if not self.__stanza_entities:
+            for entity in self.stanza_document.entities:
+                self.__stanza_entities.append(entity)
+        return self.__stanza_entities
 
 
     def __get_doc_path(self, field: str) -> str:
@@ -93,7 +122,7 @@ class Document:
         Add existing texta_facts inside the document into the private
         fact container variable so that they wouldn't be overwritten.
         """
-        if self.json_doc and "texta_facts" in self.json_doc:
+        if self.json_doc:
             existing_facts = self.json_doc.get("texta_facts", [])
             facts = Fact.from_json(existing_facts)
             for fact in facts:
@@ -174,6 +203,29 @@ class Document:
         return wrapper.convert()
 
 
+    @staticmethod
+    def parse_doc(doc_path: str, document: dict) -> list:
+        """
+        Function for parsing text values from a nested dictionary given a field path.
+        :param doc_path: Dot separated path of fields to the value we wish to parse.
+        :param document: Document to be worked on.
+        :return: List of text fields that will be processed by MLP.
+        """
+        wrapper = PelicanJson(document)
+        doc_path_as_list = doc_path.split(".")
+        content = wrapper.safe_get_nested_value(doc_path_as_list, default=[])
+        if content and isinstance(content, str):
+            return [content]
+        # Check that content is non-empty list and there are only stings in the list.
+        elif content and isinstance(content, list) and all([isinstance(list_content, str) for list_content in content]):
+            return content
+        # In case the field path is faulty and it gives you a dictionary instead.
+        elif isinstance(content, dict):
+            return []
+        else:
+            return []
+
+
     def document_to_json(self, use_default_doc_path=True) -> dict:
         """
         :param use_default_doc_path: Normal string values will be given the default path for facts but for dictionary input you already have them.
@@ -216,7 +268,7 @@ class Document:
 
     def lemmas(self):
         for sent in self.stanza_sentences:
-            self.__lemmas.append([word.lemma.replace("_", "") if word and word.lemma else "X" for word in sent])
+            self.__lemmas.append([word.lemma.replace("_", "") if word and word.lemma else "X" for word in sent.words])
 
 
     def get_lemma(self) -> str:
@@ -236,8 +288,8 @@ class Document:
 
 
     def words(self):
-        for sent in self.stanza_sentences:
-            self.__words.append([word.text for word in sent])
+        for sentence in self.stanza_sentences:
+            self.__words.append([word.text for word in sentence.words])
 
 
     def get_words(self) -> str:
@@ -250,8 +302,7 @@ class Document:
     def pos_tags(self):
         if "sentences" in self.analyzers:
             for i,sent in enumerate(self.stanza_sentences):
-                #print(sent)
-                tags_in_sent = [word.upos if word and word.upos and word.upos != "_" else "X" if word.upos == "_" else "X" for word in sent]
+                tags_in_sent = [word.upos if word and word.upos and word.upos != "_" else "X" if word.upos == "_" else "X" for word in sent.words]
                 for tag in tags_in_sent:
                     self.__pos_tags.append(tag)
                 # if not last item
@@ -268,7 +319,7 @@ class Document:
     def word_features(self):
         if "sentences" in self.analyzers:
             for i,sent in enumerate(self.stanza_sentences):
-                tags_in_sent = [word.feats if word and word.feats and word.feats != "_" else "X" if word.feats == "_" else "X" for word in sent]
+                tags_in_sent = [word.feats if word and word.feats and word.feats != "_" else "X" if word.feats == "_" else "X" for word in sent.words]
                 for tag in tags_in_sent:
                     self.__word_features.append(tag)
                 # if not last item
@@ -384,19 +435,19 @@ class Document:
         if self.dominant_language_code in Document.langs_to_transliterate:
             for word in self.stanza_words:
                 if self.dominant_language_code == "ru":
-                    translit_word = self._transliterate_russian_word(word)
+                    translit_word = self._transliterate_russian_word(word.text)
                 elif self.dominant_language_code == "ar":
-                    translit_word = self._transliterate_arabic_word(word)
+                    translit_word = self._transliterate_arabic_word(word.text)
                 self.__transliteration.append(translit_word)
 
 
     @staticmethod
-    def _transliterate_russian_word(word):
-        translit_word = russian_transliterator([word.text.strip()])
+    def _transliterate_russian_word(word: str):
+        translit_word = russian_transliterator([word.strip()])
         try:
             translit_word = translit_word[0].strip()
         except IndexError:
-            translit_word = word.text.strip()
+            translit_word = word.strip()
         return translit_word
 
 
