@@ -4,10 +4,11 @@ from typing import List
 
 class Fact:
 
-    def __init__(self, fact_type: str, fact_value: str, doc_path: str = None, fact_lemma: str = None, spans: list = [[0, 0]]):
+    def __init__(self, fact_type: str, fact_value: str, doc_path: str = None, fact_lemma: str = None, spans: list = [[0, 0]], sent_index: int = 0):
         self.fact_type = fact_type
         self.fact_value = fact_value
         self.spans = spans
+        self.sent_index = sent_index
         self.doc_path = doc_path
         self.fact_lemma = fact_lemma
 
@@ -18,6 +19,7 @@ class Fact:
         for fact_json in list_of_facts:
             fact_type = fact_json.get("fact")
             spans = json.loads(fact_json.get("spans"))
+            sent_index = fact_json.get("sent_index")
             value = fact_json.get("str_val")
             field_path = fact_json.get("doc_path")
             fact = Fact(fact_type=fact_type, fact_value=value, doc_path=field_path, spans=spans)
@@ -29,6 +31,7 @@ class Fact:
         container = {
             "str_val": self.fact_value,
             "spans": json.dumps(self.spans),
+            "sent_index": self.sent_index,
             "fact": self.fact_type,
             "doc_path": self.doc_path,
         }
